﻿using AutoMapper;
using FLY.FilmCasting.AutoDie.Common;
using FLY.FilmCasting.AutoDie.IService;
using Misc;
using Newtonsoft.Json;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;

namespace FLY.FilmCasting.AutoDie.Server
{
    /// <summary>
    /// 以前 正反转加热分离，所以有两个 HeatCell
    /// 现在不分正反转，只有一个HeatCell
    /// </summary>
    public class HeatCell : IHeatCellService
    {
        #region IHeatCell 接口 成员变量

        #region 参数
        /// <summary>
        /// HeatOffset = ThickPercent * Kp
        /// </summary>
        public double Kp { get; set; } = 1;


        /// <summary>
        /// 平滑的阀值，只有超出阀值，才平滑
        /// </summary>
        public double ThresholdHeatSigma { get; set; } = 10;


        /// <summary>
        /// 自控时，不能调整总体上升下降
        /// </summary>
        public bool IsForbidUpDown { get; set; } = false;



        /// <summary>
        /// 厚度控制线 ， 厚度% 大于 厚度控制线 自动才会加热控制  
        /// </summary>
        public double CtrlLine { get; set; } = 1;

        /// <summary>
        /// 最大加热量 默认100%
        /// </summary>
        public int MaxHeat { get; set; } = 100;
        /// <summary>
        /// 偏移量界面的显示范围
        /// </summary>
        public int RangeOfOffset { get; set; } = 50;

        /// <summary>
        /// 中心目标厚度 ,希望生产出来的产品中心比较厚点
        /// </summary>
        public double CenterTarget { get; set; }
        #endregion

        #region 状态


        /// <summary>
        /// 对应的100% 数据，单位是% 数量是ChannelCnt, 数据肯定是完整的！！！！！
        /// </summary>
        public double[] ThickPercents { get; set; }


        /// <summary>
        /// 当前加热策略改变的时间
        /// </summary>
        public DateTime LastHeatsChangedTime { get; private set; }

        /// <summary>
        /// 当前方向的加热策略，必须保证 0~100
        /// </summary>
        public double[] Heats { get; protected set; }


        private int Base { get; set; } = 0;




        /// <summary>
        /// 加热偏移量
        /// </summary>
        [DoNotCheckEquality]
        public double[] Offsets { get; protected set; }


        /// <summary>
        /// Heats+Offsets+Base
        /// </summary>
        [DoNotCheckEquality]
        public double[] PreHeats { get; protected set; }


        /// <summary>
        /// 加热量 每3个计算的sigma,最大值
        /// </summary>
        public double MaxHeatSigma { get; set; } = 0;


        #endregion

        #endregion

        public int ChannelCnt { get; set; }
        HeatBuf mHeatBuf;
        //IBlowingService blowingService;
        /// <summary>
        /// 厚度 目标值，正常情况全部都是0
        /// </summary>
        double[] thickPercentTargets;
        double centerTarget;
        int dataBegin;
        int dataEnd;

        public Func<object, bool> PreHeatApplyEvent;
        public event Action<object> AfterHeatApplyEvent;
        public event Action<object> AfterClearOffsetsEvent;


        public HeatCell()
        {
            if (!Load())
                Save();

            checkParamErr();
            Init2(88);
            
        }
        void checkParamErr()
        {
            if (Kp < 0.2 || Kp > 10)
            {
                Kp = 1;
            }

            if (ThresholdHeatSigma < 1 || ThresholdHeatSigma > 50)
            {
                ThresholdHeatSigma = 10;
            }

            if (CtrlLine < 0 || CtrlLine > 10)
            {
                CtrlLine = 2;
            }

        }
        public void Init(HeatBuf heatBuf)
        {
            mHeatBuf = heatBuf;

            this.PropertyChanged += HeatCell_PropertyChanged;
        }

        private void HeatCell_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(CenterTarget)) 
            {
                updateThickPercentTargets();
            }
        }

        //public void Init3(IBlowingService blowingService)
        //{
        //    this.blowingService = blowingService;
        //    this.blowingService.PropertyChanged += BlowingService_PropertyChanged;
        //}

        //private void BlowingService_PropertyChanged(object sender, PropertyChangedEventArgs e)
        //{
        //    //当复位区号改变了，同时也修改heatCell 的 thickPercents
        //    if (e.PropertyName == nameof(this.blowingService.OrgBoltNo))
        //    {
        //        if (mHeatBuf.MoveThickPercents(this.blowingService.OrgBoltNo))
        //            SetThickPercents(mHeatBuf.ThickPercents);
        //    }
        //}

        public void Init2(int channelCnt)
        {
            ChannelCnt = channelCnt;

            Base = 0;
            MaxHeatSigma = 0;

            ThickPercents = new double[ChannelCnt];
            Heats = new double[ChannelCnt];
            PreHeats = new double[ChannelCnt];
            Offsets = new double[ChannelCnt];
            create_thickPercentTargets();
        }
        void create_thickPercentTargets() 
        {
            create_thickPercentTargets(CenterTarget, 0, ChannelCnt - 1);
        }
        void create_thickPercentTargets(double centerTarget, int dataBegin, int dataEnd) 
        {
            if (centerTarget == 0) 
            {
                thickPercentTargets = new double[ChannelCnt];
                this.centerTarget = 0;
                this.dataBegin = 0;
                this.dataEnd = ChannelCnt - 1;
                return;
            }
            int cnt = (dataEnd - dataBegin) + 1;
            double k = Math.Sqrt(centerTarget * 2) / (cnt / 2);
            this.thickPercentTargets = new double[ChannelCnt];
            for (int i = 0; i < this.thickPercentTargets.Count(); i++)
                this.thickPercentTargets[i] = double.NaN;
            for (int i = dataBegin; i <= dataEnd; i++)
            {
                int n = i - cnt / 2;
                double t = Math.Pow(k * n, 2);
                t = centerTarget - t;
                this.thickPercentTargets[i] = Math.Round(t, 2);
            }
            this.centerTarget = centerTarget;
            this.dataBegin = dataBegin;
            this.dataEnd = dataEnd;
        }
        
        /// <summary>
        /// 自动模式！！！！！
        /// </summary>
        public void Auto()
        {
            if (mHeatBuf.mData.Count() == 0)
                return;
            //&& mHeatBuf.IsStable()

            if (mHeatBuf.Stability == STABILITY.OK_CORREL)//稳定 与 待定时 加热
            {
                //转为 加热棒数量
                var thickPercents = ThickPercents;

                double[] offsets = new double[ChannelCnt];
                for (int i = 0; i < ChannelCnt; i++)
                {
                    if (!double.IsNaN(thickPercents[i]) && !double.IsNaN(thickPercentTargets[i]) &&
                        mHeatBuf.BoltIsStable[i] && //稳定
                        (Math.Abs(thickPercents[i]) > CtrlLine))//在控制线已外
                    {
                        offsets[i] = (thickPercents[i] - thickPercentTargets[i]) * Kp;//保留一位小数
                    }
                }

                Offsets = offsets;
                UpdatePreHeats();//这句一定要加！！！！ 不然没法平滑，没法加热

                if (!IsForbidUpDown)//禁止整体上移下降
                    ModifyBase();//自动整体上移下降

                Smooth();//平滑,它会导致Offset被修改，<0的都被删除掉

                HeatApply();//执行加热
            }
            
        }


        public void SetThickPercents(double[] thkpercents)
        {
            ThickPercents = thkpercents.ToArray();
            updateThickPercentTargets();
        }
        void updateThickPercentTargets() 
        {
            var centerTarget = Math.Round(CenterTarget, 1);
            if(centerTarget == 0)
            {
                create_thickPercentTargets();
                return;
            }
            
            //找到 ThickPercents 有效数据的开始， 与结束
            int dataBegin =-1;
            for (int i = 0; i < ThickPercents.Count(); i++) 
            {
                if (!double.IsNaN(ThickPercents[i]))
                {
                    dataBegin = i;
                    break;
                }
            }
            if (dataBegin == -1) {
                //全部数据无效
                return;
            }
            int dataEnd = -1;
            for (int i = ThickPercents.Count()-1; i >=0; i--)
            {
                if (!double.IsNaN(ThickPercents[i]))
                {
                    dataEnd = i;
                    break;
                }
            }
            if (dataEnd == -1)
            {
                //全部数据无效
                return;
            }

            //创建目标曲线
            create_thickPercentTargets(centerTarget, dataBegin, dataEnd);
        }

        #region IHeatCell 接口 成员函数



        /// <summary>
        /// 手动调整加热增量
        /// </summary>
        /// <param name="idx"></param>
        /// <param name="change"></param>
        public void ModifyOffset(int idx, int change)
        {
            if (idx < ChannelCnt && idx >= 0)
            {
                Offsets[idx] += change;
                UpdatePreHeats();
            }
            NotifyPropertyChanged(nameof(Offsets));
        }

        /// <summary>
        /// 参数是 baseheatoffset 的增量
        /// </summary>
        /// <param name="change"></param>
        public void ModifyBase(int change)
        {
            Base += change;
            UpdatePreHeats();
        }

        /// <summary>
        /// 调整BaseHeatOffset，令Heats 在范围内
        /// </summary>
        void ModifyBase()
        {
            double[] preheat = new double[ChannelCnt];
            for (int i = 0; i < ChannelCnt; i++)
                preheat[i] = Heats[i] + Offsets[i] + Base;

            int min = (int)preheat.Min();
            int max = (int)preheat.Max();
            int avg = (int)preheat.Average();

            int base_offset;
            int maxHeat = MaxHeat;

            if ((min < 0) && (max > maxHeat))
            {
                //功率极限了
                //保存平均值为50
                base_offset = avg - maxHeat / 2;
            }
            else if (max > maxHeat)
            {

                //整体往下降
                base_offset = max - maxHeat;

                if (base_offset > (avg - maxHeat / 2))//移得太多
                    base_offset = (avg - maxHeat / 2);
            }
            else
            {
                base_offset = min;
                if (base_offset < 0)
                {
                    if (base_offset < (avg - maxHeat / 2))//升得太多
                        base_offset = (avg - maxHeat / 2);
                }
            }

            Base -= base_offset;

            UpdatePreHeats();
        }

        /// <summary>
        /// 清空HeatOffset
        /// </summary>
        public void ClearOffsets()
        {
            Base = 0;

            for (int i = 0; i < ChannelCnt; i++)
            {
                Offsets[i] = 0;
            }
            NotifyPropertyChanged(nameof(Offsets));
            UpdatePreHeats();

            AfterClearOffsetsEvent?.Invoke(this);
        }

        /// <summary>
        /// 清空Heats
        /// </summary>
        public void ClearPreHeats()
        {
            ModifyPreHeats(new double[ChannelCnt]);
        }

        /// <summary>
        /// 调整PreHeats
        /// </summary>
        /// <param name="heats"></param>
        public void ModifyPreHeats(double[] heats)
        {

            if (heats.Count() != ChannelCnt)
                return;
            Base = 0;
            for (int i = 0; i < ChannelCnt; i++)
            {
                PreHeats[i] = heats[i];
                Offsets[i] = PreHeats[i] - Heats[i];
            }
            NotifyPropertyChanged(nameof(Offsets));
            NotifyPropertyChanged(nameof(PreHeats));
            UpdateHeatSigmas();
        }
        public void ModifyHeats(double[] heats)
        {
            if (heats.Count() != ChannelCnt)
                return;

            bool isChanged = false;
            Base = 0;
            for (int i = 0; i < ChannelCnt; i++)
            {
                var h = Math.Round(heats[i], 1);
                if (Heats[i] != h)
                {
                    isChanged = true;
                    Heats[i] = h;
                    Offsets[i] = PreHeats[i] - Heats[i];
                }
            }
            if (isChanged)
            {
                NotifyPropertyChanged(nameof(Offsets));
                NotifyPropertyChanged(nameof(Heats));
                LastHeatsChangedTime = DateTime.Now;
            }
        }
        /// <summary>
        /// 利用测厚仪的数据，计算出HeatOffsets
        /// </summary>
        public void Cal()
        {
            var thickpercents = ThickPercents;
            double[] offsets = new double[ChannelCnt];
            for (int i = 0; i < ChannelCnt; i++)
            {
                if (double.IsNaN(thickpercents[i]) || double.IsNaN(thickPercentTargets[i]))
                    offsets[i] = 0;
                else
                    offsets[i] = (thickpercents[i] - thickPercentTargets[i]) * Kp;
            }
            Offsets = offsets;
            UpdatePreHeats();
        }
        #region PreHeats 平滑处理
        double[] GetHeatSigmas(double[] heats)
        {
            var heatsigmas = new double[ChannelCnt];
            for (int i = 0; i < ChannelCnt; i++)
            {
                var dat = getSubArray(heats, i);
                heatsigmas[i] = dat.Sigma();
            }
            return heatsigmas;
        }
        void UpdateHeatSigmas()
        {
            var heatsigmas = GetHeatSigmas(PreHeats);
            MaxHeatSigma = heatsigmas.Max();
        }

        T[] getSubArray<T>(T[] array, int index)
        {
            T[] dat = new T[3];
            for (int j = 0; j < 3; j++)
            {
                int idx = index - 1 + j;
                if (idx < 0)
                    idx += array.Count();
                else if (idx >= array.Count())
                    idx -= array.Count();

                dat[j] = array[idx];
            }
            return dat;
        }
        /// <summary>
        /// 平滑，不断修改PreHeats 直到HeatsSigma 合格
        /// </summary>
        public void Smooth()
        {
            double[] heats = PreHeats.ToArray();
            double[] heats2 = new double[ChannelCnt];
            bool hasChanged2 = false;
            double[] heatsigmas;
            while(true)
            {
                //1.更新sigma
                heatsigmas = GetHeatSigmas(heats);

                //2.检测HeatsSigma， 哪里比较大，对该位置 滤波
                for (int i = 0; i < heats.Count(); i++)
                {
                    if (heatsigmas[i] > (ThresholdHeatSigma)) //不行，要滤波！！！
                    {
                        //连续3个滤波
                        double[] dat = getSubArray(heats, i);
                        heats2[i] = Math.Round(dat.Average(), 1);
                    }
                    else
                    {
                        heats2[i] = heats[i];
                    }
                }

                //3.有滤波行为，继续求循环，求sigma，判断。。。。。
                bool hasChanged = false;
                
                for (int i = 0; i < heats.Count(); i++)
                {
                    if (heats[i] != heats2[i])
                    {
                        heats[i] = heats2[i];
                        hasChanged = true;
                    }
                }
                if (hasChanged)
                    hasChanged2 = true;
                else
                    break;
                
            }

            if (hasChanged2)
            {
                for (int i = 0; i < ChannelCnt; i++)
                {
                    PreHeats[i] = heats[i];
                    Offsets[i] = PreHeats[i] - Heats[i] - Base;
                }
                NotifyPropertyChanged(nameof(PreHeats));
                NotifyPropertyChanged(nameof(Offsets));
                MaxHeatSigma = heatsigmas.Max();
            }
        }
        #endregion
        /// <summary>
        /// 加热输出应用
        /// </summary>
        public void HeatApply()
        {
            if (PreHeatApplyEvent != null)
            {
                if (!PreHeatApplyEvent(this))
                {
                    return;
                }
            }

            for (int i = 0; i < ChannelCnt; i++)
            {
                Offsets[i] = 0;

                var h = Math.Round(PreHeats[i], 1);
                
                if (h < 0)
                {
                    //需要整体提升
                    h = 0;
                }
                else if (h > 100)
                {
                    //需要整体调低
                    h = 100;
                }
                Heats[i] = h;

            }
            NotifyPropertyChanged(nameof(Offsets));
            NotifyPropertyChanged(nameof(Heats));
            LastHeatsChangedTime = DateTime.Now;
            Base = 0;

            AfterHeatApplyEvent?.Invoke(this);
        }

        /// <summary>
        /// 修改Offsets, Base 后，执行PreHeats = Heats + Offsets + Base
        /// </summary>
        void UpdatePreHeats()
        {
            for (int i = 0; i < ChannelCnt; i++)
            {
               double h = Heats[i] + Offsets[i] + Base;

                if (h > 100)
                    h = 100;
                else if (h < 0)
                    h = 0;
                PreHeats[i] = h;
            }
            UpdateHeatSigmas();
            NotifyPropertyChanged(nameof(PreHeats));
        }

        public void Apply()
        {
            Save();
        }
        #endregion


        string file_path = "heatcell.json";
        bool Load()
        {
            if (File.Exists(file_path))
            {
                try
                {
                    string json = File.ReadAllText(file_path);
                    var jsonDb = JsonConvert.DeserializeObject<HeatCellJsonDb>(json);
                    HeatCellJsonDb.Mapper.Map(jsonDb, this);
                    return true;
                }
                catch
                {
                    //异常，没有json 解码失败

                }
                return false;
            }
            return false;
        }
        void Save()
        {
            try
            {
                var jsonDb = HeatCellJsonDb.Mapper.Map<HeatCellJsonDb>(this);
                string json = JsonConvert.SerializeObject(jsonDb, Formatting.Indented);
                File.WriteAllText(file_path, json);
            }
            catch
            {
                //异常，没有json 编码失败

            }
        }

        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }


        #endregion
    }
    public class HeatCellJsonDb
    {
        public static Mapper Mapper { get; } = new AutoMapper.Mapper(new MapperConfiguration(c =>
        {
            c.CreateMap<HeatCell, HeatCellJsonDb>().ReverseMap();
        }));

        #region 参数
        /// <summary>
        /// HeatOffset = ThickPercent * Kp
        /// </summary>
        public double Kp = 1;


        /// <summary>
        /// 自动对位
        /// </summary>
        public bool IsAutoONo = true;

        /// <summary>
        /// 最大加热量
        /// </summary>
        public int MaxHeat = 100;

        /// <summary>
        /// 偏移量界面的显示范围
        /// </summary>
        public int RangeOfOffset = 50;

        /// <summary>
        /// 平滑的阀值，只有超出阀值，才平滑
        /// </summary>
        public double ThresholdHeatSigma = 10;


        /// <summary>
        /// 自控时，不能调整总体上升下降
        /// </summary>
        public bool IsForbidUpDown = false;



        /// <summary>
        /// 厚度控制线 ， 厚度% 大于 厚度控制线 自动才会加热控制  
        /// </summary>
        public double CtrlLine = 1;

        public double CenterTarget = 0;

        #endregion
    }
}
